package Net::OATH::Server;
use 5.008001;
use strict;
use warnings;

our $VERSION = "0.02";

1;
__END__

=encoding utf-8

=head1 NAME

Net::OATH::Server - Library for One Time Password Server

=head1 DESCRIPTION

Net::OATH::Server is library for One Time Password Server.

=head1 SEE ALSO

L<Net::OATH::Server::Lite>

=head1 LICENSE

Copyright (C) ritou.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 AUTHOR

ritou E<lt>ritou.06@gmail.comE<gt>

=cut

